      subroutine cutoff(ngal,nl,r,h,ierr)
      
c     This subroutine calculates the cutoff radius for the Ellipsefitting &
c     moving the centrum
c     
c     the criteria for ellipsefitting are:
c     rshap - this has  highest priority if not 0
c     errshap - if rshap != 0 then the program calculates the radius
c       where err(second harmonical)=errshap - this will be the cutoff radius
c
c     the criteria for moving the centrum of the ellipse are
c     rcen - this has highest priority
c     errcen - if rcen != 0 then the program calculates the radius
c       where err(first harmonical)=errcen - this will be the cutoff radius
c
      include 'param.inc'
      include 'declare.inc'
      integer nl(MAXGAL)

      real error(MAXLINE),mederror(MAXLINE)
      character*128 string

      call startsub('CUTOFF')

      do 10 igal=1,ngal

         if (rshap.ne.0.) then
            do 20 il=1,nl(igal)
               if (r(1,il,igal).gt.rshap) then
                  nl(igal)=il
                  goto 21
               end if
 20         continue
 21         continue
         else if (errshap.ne.0.) then
c     calculate the radius at which error(second harmonical)=errshap
            
            do 30 il=1,nl(igal)
               error(il)=sqrt(h(2,1,2,il,igal)**2+h(2,2,2,il,igal)**2)
               if (error(il).eq.0.) error(il)=9.999
 30         continue
            
            nwin=5
            call vecmedian(error,nl(igal),nwin,mederror,ierr)
            if (ierr.ne.0) return
            
            if (debug.ge.4) then
               do 31 il=1,nl(igal)
                  write(string,'('' CUTOFF: r,e,med(e) : '',3f10.3)')
     $                 r(1,il,igal),error(il),mederror(il)
                  call gwrits(string,1)
 31            continue
            end if

            call errgetrad(igal,nl,mederror,errshap,imax,ierr)
            if (ierr.ne.0) return
            
            if (debug.ge.2) then
               write(string,
     $            '('' CUTOFF: errshap, imax, r,r(nl), med(error) :'',
     $            f6.3,i5,2f6.1,f6.3)') errshap, imax, r(1,imax,igal),
     $            r(1,nl(igal),igal),mederror(imax)
               call gwrits(string,1)
            end if
            rshap=r(1,imax,igal)
            nl(igal)=imax
         end if
         
         
         if (errcen.ne.0.) then
c     calculate the radius at which error(first harmonical)=errcen
            
            do 40 il=1,nl(igal)
               error(il)=sqrt(h(2,1,1,il,igal)**2+h(2,2,1,il,igal)**2)
               if (error(il).eq.0.) error(il)=9.999
 40         continue
            
            nwin=5
            call vecmedian(error,nl(igal),nwin,mederror,ierr)
            if (ierr.ne.0) return
            
            call errgetrad(igal,nl,mederror,errcen,imax,ierr)
            if (ierr.ne.0) return
            
            if (debug.ge.2) then
               write(string,
     $            '('' CUTOFF: errcen, imax, r,r(nl), med(error) :'',
     $            f6.3,i5,2f6.1,f6.3)') errcen, imax, r(1,imax,igal),
     $            r(1,nl(igal),igal),mederror(imax)
               call gwrits(string,1)
            end if
            rcen=r(1,imax,igal)
         end if
         
 10   continue
      
      ierr=0
      return
      end

      subroutine errgetrad(igal,nl,error,errmax,imax,ierr)

      include 'param.inc'
      integer  nl(MAXGAL)
      real error(MAXLINE)

      character*128 string
c     first we search for the radius at which the error is minimal

      imin=1
      err=error(1)
      do 10 i=2,nl(igal)
         if (error(i).lt.err) then
            imin=i
            err=error(i)
         end if
 10   continue

c     we found the minimum. If the minimum > errmax we are in trouble !

      if (err.gt.errmax) then
         write(string,
     $        '('' CUTOFF: ERROR: minimum error > errmax ! '',2g10.4)')
     $        err,errmax
         call gwrits(string,2)
         ierr=1
         return
      end if

c     now we start at i=imin and look for the first occurrence of
c     error > errmax

      do 20 i=imin,nl(igal)
         if (error(i).gt.errmax) then
            imax=i-1
            return
         end if
 20   continue
      
      imax=nl(igal)

      ierr=0
      return
      end
