      subroutine square(nx, ny, pix, accurate, x, y, value)
      
c     subroutine gets mean pixel value within a area defined by
c     4 points. Only the pixels which lie within the area are included;
c     no nice cutoff near the edges

      include 'param.inc'
      
      real 	pix(nx,ny), value
      real	x(5), y(5)
      
      integer	inter(2,10),interc(2,10)
      logical	accurate
      character*128	string
      
      value=0.
      area=0.
      bad=0.
      xsum=0.
      ysum=0.
      
      xmin=32000
      xmax=-32000
      ymin=32000
      ymax=-32000
      
      do 10 i=1,4
         if (xmin.gt.x(i)) xmin=x(i)	
         if (xmax.lt.x(i)) xmax=x(i)	
         if (ymin.gt.y(i)) ymin=y(i)	
         if (ymax.lt.y(i)) ymax=y(i)	
 10   continue
      
      if (xmax.lt.0.or.xmin.gt.nx.or.ymax.lt.0.or.ymin.gt.ny) then
         value=VALBAD
         x(5)=0.
         y(5)=0.
         return
      end if
      
      x(5)=x(1)
      y(5)=y(1)
      
      nx1=xmin
      nx2=xmax+1.
      ny1=ymin
      ny2=ymax+1.
      
      if (nx1.lt.1 ) nx1=1
      if (nx2.gt.nx) nx2=nx
      if (ny1.lt.1)  ny1=1
      if (ny2.gt.ny) ny2=ny
      
      do 20 iy=ny1,ny2
         if (accurate) then
            call intsec(5,x,y,iy,ninter,inter,nx1,nx2)
            if (debug.ge.20) then
               write(string,'(''SQUARE: '',3i5)')
     &              ninter,inter(1,1),inter(2,1)
               call gwrits(string,1)
            end if
         else
            inter(1,1)=nx1
            inter(2,1)=nx2
            ninter=1
         end if
         
         do 21 i=1,ninter
            do 22 ix=inter(1,i),inter(2,i)
               
               if (pix(ix,iy).gt.VALMIN) then
		  value=value+pix(ix,iy)
		  area=area+1.
		  xsum=xsum+float(ix)
		  ysum=ysum+float(iy)
               else
		  bad=bad+1.
               end if
               if (debug.ge.21) then
                  write(string,'(''SQUARE: '',g12.4,2f8.0)')
     &                 pix(ix,iy),area,bad
                  call gwrits(string,1)
               end if
 22         continue
            
 21      continue
 20   continue
      
      if (area.gt.0.8*bad) then
         value=value/area
         x(5)=xsum/area
         y(5)=ysum/area
      else
         value=VALBAD
         x(5)=x(1)
         x(5)=y(2)
      end if
      
      return
      end
      
      subroutine intsec(n,x,y,iy,ninter,inter,nx1,nx2)
      
c     subroutine calculates the interval(s) in x at fixed iy
c     which lie inside the area defined by x(n) y(n)
c     someaht obscure algorithm, but tested well

      real	x(n),y(n)
      integer inter(2,10)
      
      real	xinter(10)
      include 'param.inc'
      logical low,high
      
      yval=iy
      xmin=nx1-0.01
      xmax=nx2+0.01
      xinter(1)=xmin
      xinter(2)=xmax
      nxint=2
      
c     CALCULATE WHERE BORDERLINES INTERSEC LINE Y=IY

      do 10 i=1,n-1
         dx=x(i+1)-x(i)
         dy=y(i+1)-y(i)
         if (abs(dy).ge.1e-10) then
            xint=x(i)+dx*(yval-y(i))/dy
            if (xint.gt.xmin.and.xint.lt.xmax) then
               nxint=nxint+1
               xinter(nxint)=xint
            end if
         end if
 10   continue
      
C     SORT THE INTERSECTIONS
      do 20 i=1,nxint-1
         do 21 j=i+1,nxint
            if (xinter(j).lt.xinter(i)) then
               t=xinter(j)
               xinter(j)=xinter(i)
               xinter(i)=t
            end if
 21      continue
 20   continue
      
C     THROW OUT DOUBLE INTERSECTIONS
      in=1
      iout=1
 30   if (in.lt.nxint) then
         if (xinter(in).ne.xinter(in+1)) then
            in=in+1
            iout=iout+1
            xinter(iout)=xinter(in)
         else
            in=in+1
         end if
         goto 30
      end if
      nxint=iout
      
C     CHECK FOR EACH INTERSECTION IF A SINGLE POINT LIES ON IT &
C     IF IT DOES, CALCULATE FOR ONE SINGLE POINT WHETHER IT IS
C     INSIDE OR OUTSIDE THE AREA

      ninter=0
      do 40  i=1,nxint-1
         ix1=int(xinter(i))+1
         ix2=int(xinter(i+1))
         if (ix1.le.ix2) then
            low=.true.
            high=.true.
            do 41 j=1,n-1
               if (low.or.high) then
                  diff=(y(j+1)-y(j))*(ix1-x(j))-
     &                 (x(j+1)-x(j))*(yval-y(j))
                  low=low.and.(diff.lt.0.)
                  high=high.and.(diff.gt.0.)
               end if
 41         continue
            if (low.or.high) then
               ninter=ninter+1
               inter(1,ninter)=ix1
               inter(2,ninter)=ix2
            end if
         end if
 40   continue
      
      
      end
      subroutine intsim(n,x,y,iy,ninter,inter,nx1,nx2)
      
c     subroutine calculates the interval(s) in x which lie inside
c     the area defined by x(n) y(n)
c     simple algorimth, but SLOW. Do not use !

      real	x(n),y(n)
      integer inter(2,10)
      
      include 'param.inc'
      
      logical low,high
      
      is=0
      ninter=0
      
      do 10 ix=nx1,nx2
         low=.true.
         high=.true.
         do 41 j=1,n-1
            if (low.or.high) then
               diff=(y(j+1)-y(j))*(ix-x(j))-
     &              (x(j+1)-x(j))*(iy-y(j))
               low=low.and.(diff.lt.0.)
               high=high.and.(diff.gt.0.)
            end if
 41      continue
         if (low.or.high) then
            if (is.eq.0) is=ix
         else
            if (is.ne.0) then	
               ninter=ninter+1
               inter(1,ninter)=is
               inter(2,ninter)=ix-1
               is=0
            end if
         end if
 10   continue
      
      if (is.ne.0) then	
         ninter=ninter+1
         inter(1,ninter)=is
         inter(2,ninter)=nx2
         is=0
      end if
      
      return
      end
